package com.iformall.mp.controller;

import com.iformall.mp.manager.WxMpManager;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutMessage;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.nio.charset.Charset;

/**
 * @author Stormeye
 */
@Controller
@RequestMapping("/wx")
public class WxMpCallBackController {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private WxMpManager wxMpManager;

    /**
     * GET请求：进行URL、Tocken 认证；
     * 1. 将token、timestamp、nonce三个参数进行字典序排序
     * 2. 将三个参数字符串拼接成一个字符串进行sha1加密
     * 3. 开发者获得加密后的字符串可与signature对比，标识该请求来源于微信
     */
    @GetMapping(value = "/{account}/message")
    @ResponseBody
    public void authGet(@PathVariable String account, HttpServletRequest request, HttpServletResponse response) throws IOException {
        String signature = request.getParameter("signature");// 微信加密签名
        String timestamp = request.getParameter("timestamp");// 时间戳
        String nonce = request.getParameter("nonce");// 随机数
        String echostr = request.getParameter("echostr");// 随机字符串

        if (StringUtils.isAnyBlank(signature, timestamp, nonce, echostr)) {
            throw new IllegalArgumentException("请求参数非法，请核实!");
        }

        logger.info("接收到来自微信服务器的认证消息：[{}, {}, {}, {}]",
            signature, timestamp, nonce, echostr);

        // 数据库 ->
        final WxMpService wxService = wxMpManager.getMpService(account);
        if (wxService == null) {
            throw new IllegalArgumentException(String.format("未找到对应account=[%s]的配置，请核实！", account));
        }

        if (wxService.checkSignature(timestamp, nonce, signature)) {
            logger.info("微信服务验证成功");
            PrintWriter out = response.getWriter();
            out.print(echostr);
            out.close();
        }
    }

    /**
     * POST 请求：进行消息处理；
     */
    @PostMapping(value = "/{account}/message")
    @ResponseBody
    public String post(@PathVariable String account, HttpServletRequest request) throws IOException {
        // 处理用户和微信公众账号交互消息
        String signature = request.getParameter("signature");// 微信加密签名
        String timestamp = request.getParameter("timestamp");// 时间戳
        String nonce = request.getParameter("nonce");// 随机数
        String openid = request.getParameter("openid");
        String encType =  request.getParameter("encrypt_type");
        String msgSignature = request.getParameter("msg_signature");

        if (StringUtils.isAnyBlank(signature, timestamp, nonce)) {
            throw new IllegalArgumentException("请求参数非法，请核实!");
        }

        final WxMpService wxService = wxMpManager.getMpService(account);

        if (!wxService.checkSignature(timestamp, nonce, signature)) {
            throw new IllegalArgumentException("非法请求，可能属于伪造的请求！");
        }

        InputStream inStream = request.getInputStream();
        ByteArrayOutputStream outSteam = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int len = 0;
        while ((len = inStream.read(buffer)) != -1) {
            outSteam.write(buffer, 0, len);
        }
        String requestBody = new String(outSteam.toByteArray(), Charset.forName("UTF-8"));
        logger.info(requestBody);

        outSteam.close();
        inStream.close();

        logger.info("\n接收微信请求：[openid=[{}], [signature=[{}], encType=[{}], msgSignature=[{}],"
                + " timestamp=[{}], nonce=[{}], requestBody=[\n{}\n] ",
            openid, signature, encType, msgSignature, timestamp, nonce, requestBody);

        String out = null;
        if (encType == null) {
            // 明文传输的消息
            WxMpXmlMessage inMessage = WxMpXmlMessage.fromXml(requestBody);

            WxMpXmlOutMessage outMessage = this.route(inMessage, wxService);
            if (outMessage == null) {
                logger.info("1 return: success");
                return "success";
            }

            out = outMessage.toXml();
            logger.info(out);
        } else if ("aes".equalsIgnoreCase(encType)) {
            // aes加密的消息
            WxMpXmlMessage inMessage = WxMpXmlMessage.fromEncryptedXml(requestBody, wxService.getWxMpConfigStorage(),
                timestamp, nonce, msgSignature);
            this.logger.debug("\n消息解密后内容为：\n{} ", inMessage.toString());
            WxMpXmlOutMessage outMessage = this.route(inMessage, wxService);
            if (outMessage == null) {
                logger.info("2 return: success");
                return "success";
            }
            logger.info("out: " + outMessage.toXml());

            out = outMessage.toEncryptedXml(wxService.getWxMpConfigStorage());
        }
        this.logger.info("\n组装回复信息：{}", out);
        return out;
    }

    private WxMpXmlOutMessage route(WxMpXmlMessage message, WxMpService wxService) {
        String appId = wxService.getWxMpConfigStorage().getAppId();
        logger.info("route appId: " + appId);
        try {
            return wxMpManager.getRouter(appId).route(message);
        } catch (Exception e) {
            this.logger.error("路由消息时出现异常！", e);
        }

        return null;
    }

}
